@@ -19,26 +19,26 @@ sub parse {
         version
     )) {
         next unless '' ne ($t = $xp -> findvalue('@'.lc $o, $node));
-        $new_options{$o} = $t;
+        $new_options{$o} = "".$t;
     }
 
     foreach my $o (qw(
         password
     )) {
         next unless '' ne ($t = $xp -> findvalue('@'.lc $o, $node));
-        $bind_options{$o} = $t;
+        $bind_options{$o} = "".$t;
     }
 
-    my $host = $xp -> findvalue('@host', $node);
+    my $host = "".$xp -> findvalue('@host', $node);
 
     my $inst = ResourcePool::Factory::Net::LDAP->new($host, %new_options);
 
-    my $dn = $xp -> findvalue('@dn', $node);
+    my $dn = "".$xp -> findvalue('@dn', $node);
     $inst -> bind($dn, %bind_options);
 
     my %lb_options;
-    $lb_options{Weight} = $xp -> findvalue('@weight', $node);
-    $lb_options{SuspendTimeout} = $xp -> findvalue('@suspendtimeout', $node) || 5;
+    $lb_options{Weight} = "".$xp -> findvalue('@weight', $node);
+    $lb_options{SuspendTimeout} = "".$xp -> findvalue('@suspendtimeout', $node) || 5;
     my %rp_options;
     foreach my $o (qw(
         Max
@@ -46,7 +46,7 @@ sub parse {
         SleepOnFail
     )) {
         next unless '' ne ($t = $xp -> findvalue('@'.lc $o, $node));
-        $rp_options{$o} = $t;
+        $rp_options{$o} = "".$t;
     }
 
     $inst = ResourcePool -> new($inst, %rp_options);
